#### Chr 3 特征选取（原文用的弹性网络），我们这里用Lasso和随机森林 ####

### 随机森林（生存分析) ###
library(tidyverse)
library(survivalsvm) ## 加载R包
library(randomForestSRC)
getwd()
setwd("D:/LUAD/LUAD_M6A")
load("./data/TCGA_LUAD.Rdata")

#### 0.2 
load("./results/1-单因素cox/UniVar_0.2.Rdata")
load("./results/六队列共有基因.Rdata")
dir.create("results/2-Lasso&RSF")
df <- TCGA_LUAD

####  0.2
Factors <- intersect(UniVar_0.2,gene_intersect)
df <- subset(df,select = c('OS_time', 'OS_status',Factors))

out.rsf.1 <- rfsrc(Surv(OS_time, OS_status) ~ . ,
                   data = df ,
                   ntree = 1000, ## tree的个数
                   nsplit = 1) ## 随机拆分数（非负整数），可提高运算速度，默认值为0

print.rfsrc(out.rsf.1) ## 输出结果信息

pred <- predict(out.rsf.1,
                df,
                OOB=TRUE,
                type="response")

gene.vs<-var.select(object = out.rsf.1,
                    method="vh") ## two options : 'vh':速度慢，准确度高；'md':速度快，准确度相对较低。

randomForestSRC_geneids <- gene.vs$topvars ## 提取分析结果中方差较大的变量
dir.create("./results/2-Lasso&RSF/RSF")
save(randomForestSRC_geneids,file = './results/2-Lasso&RSF/RSF/Chr3_Results_rfsrc_genes.RData')

### Lasso Cox ###
library("glmnet") ## 加载R包
dir.create("./results/2-Lasso&RSF/Lasso")
expr_df <- df[,-c(1:2)] %>% as.matrix() ## 根据R包的要求，将数据需要筛选的部分提取转换为矩阵
cvfit = cv.glmnet(expr_df,
                  Surv(df$OS_time,df$OS_status), 
                  nfold=100,
                  family = 'cox'
) 

pdf('./results/2-Lasso&RSF/Lasso/Plot3_Lasso_1.pdf')
plot(cvfit) ## 画图
dev.off()
str(df$OS_time)
fit <- glmnet(expr_df, Surv(df$OS_time,df$OS_status), 
              family = "cox") 

pdf('./results/2-Lasso&RSF/Lasso/Plot3_Lasso_2.pdf')
plot(fit, label = TRUE)
dev.off()

coef.min = coef(cvfit, s = "lambda.min")  ## lambda.min & lambda.1se

active.min = which(coef.min != 0 ) ## 找出那些回归系数没有被惩罚为0的

(lasso_geneids <- colnames(expr_df)[active.min]) ## 提取基因名称


save(lasso_geneids,file = './results/2-Lasso&RSF/Lasso/Chr3_Results_lasso_geneids.RData')


### Venn Plot ###

load("./results/2-Lasso&RSF/RSF/Chr3_Results_rfsrc_genes.RData")
load("./results/2-Lasso&RSF/Lasso/Chr3_Results_lasso_geneids.RData")

Final_genes <- intersect(lasso_geneids,randomForestSRC_geneids)

library(VennDiagram)
pdf('./results/2-Lasso&RSF/venn.pdf')
venn.plot <- venn.diagram(
  x = list(
    LASSO = lasso_geneids, ## lasso筛选的结果
    RandomForestSRC = randomForestSRC_geneids ## 随机森林筛选的结果
  ),
  filename = paste0('Plot3_lasso_RFSRC_sur_venn.eps'), ## 输出图片的名字
  lwd = 3, ## 线的粗细
  fill = c("cornflowerblue", "darkorchid1"), ## 输出图像的颜色
  alpha = 0.6, ## 透明度
  label.col = "black", ## label的颜色
  cex = 1.5, ## 字符大小
  fontfamily = "serif", ## 字符格式
  fontface = "bold", ## 是否加粗
  cat.col = c("cornflowerblue", "darkorchid1"), ## 图例的颜色
  cat.cex = 2, ## 图例字体大小
  cat.fontfamily = "serif", ## 字体格式
  cat.fontface = "bold", ## 是否加粗
  margin = 0.05,
  cat.dist = c(0.03, 0.03),
  cat.pos = c(-20, 20)
)
dev.off()
getwd()



## 将上述Hub特征基因纳入多因素cox分析  本次训练没有用
Surv(df$OS_time,df$OS_status)
fml <-  as.formula(paste0('Surv(df$OS_time,df$OS_status)~',paste0(Final_genes,collapse = '+'))) 
MultiCox <- coxph(fml, data = df) ## 多因素Cox回归
MultiSum <- summary(MultiCox) ## 提取分析结果

MultiName <- as.character(Final_genes)
MHR <- round(MultiSum$coefficients[,2],2)
MPV <- round(MultiSum$coefficients[,5],3)
MCIL <- round(MultiSum$conf.int[,3],2)
MCIU <- round(MultiSum$conf.int[,4],2)
MCI <- paste0(MCIL,'-',MCIU)
MulCox <- data.frame('characteristics' = MultiName, ## 构建多因素cox分析结果数据框
                     'Hazard Ratio' = MHR,
                     'CI95' = MCI,
                     'P Value' = MPV)

(Final_genes_select <- MulCox$characteristics[MulCox$P.Value < 0.05])

save(Final_genes_select,file = './results/2-Lasso&RSF/Chr3_Results_Final_genes.RData')
