library(tidyverse)
library(ggplot2)
library(survminer)
library(survival)
library(timeROC)
rm(list = ls())
getwd()
setwd("D:/LUAD/LUAD_M6A")

load("./data/TCGA_LUAD.Rdata")
load("./data/GSE31210.Rdata")
load("./data/GSE30219.RData")
load("./data/GSE31546.RData")
load("./data/GSE37745.RData")
load("./data/GSE50081.RData")
load("./data/GSE68465.RData")

load("./results/1-单因素cox/UniVar.Rdata") 
setwd("D:/LUAD/LUAD_TMB")
dir.create("./results/KM+timeROC")
# filter_output <- read_csv("./results/filter_output.csv")
name <- c("GSE31210","TCGA_LUAD","GSE30219")
list <- list(GSE31210,TCGA_LUAD,GSE30219)

load("D:/LUAD/LUAD_M6A/1116_LUAD_RESULTS_GSE31210训练_3对垒/df_coef/GSE30219_df_coef.rdata")
for (i in 1:length(filter_output$file)) {

gene<-unlist(strsplit(filter_output$multi_genes[i],'_'))
# project <- filter_output$file[i]
# dir.create(paste0("./results/KM+timeROC/",project))
for (j in 1:length(name)) {
df <- data.frame(list[j])

load(file =paste0("./results/df_coef/",name[j],"_df_coef.rdata"))
df_coef <- select(df_coef,gene)
signature <- rowSums(df_coef)
df$signature <- signature


## KM分析进行验证 ##
LUAD_RISKSCORE <- subset(LUAD_risk,select = c("OS_status","OS_time","risk_score"))
LUAD_RISKSCORE$ID <- rownames(LUAD_RISKSCORE)
LUAD_TMB_riskscore <- merge(LUAD_TMB_filtersample, LUAD_RISKSCORE, by="ID")
LUAD_TMB_riskscore <- LUAD_TMB_riskscore[,c(1,3,5,6)]
rownames(LUAD_TMB_riskscore) <- LUAD_TMB_riskscore$ID
LUAD_TMB_riskscore <- LUAD_TMB_riskscore[,-1]

df <- LUAD_TMB_riskscore
colnames(df)[1] <- "signature"
df$signature_by2 <- ifelse(df$signature > median(df$signature),'High-Score','Low-Score') 
## 这里的命名要注意！！！

fit <- survfit(Surv(OS_time, OS_status)~signature_by2,data = df)

p <- ggsurvplot(fit, conf.int=F, pval=T, risk.table=T, legend.labs = c('High-Score','Low-Score'),
                legend.title='Risk Score', palette = c("dodgerblue2","orchid2"),
                risk.table.height = 0.3)
p

pdf("./TMB_KMplot.pdf",onefile = F,width = 8,height = 6)
print(p)
dev.off()





#### Chr 6 Time-ROC进行验证 ####


cox.timp2 <- coxph(Surv(df$OS_time, df$OS_status == 1) ~ df$signature_by2)

lpFit <- cox.timp2$linear.predictors


# library(survival)
# # Time-dependent ROC curve estimation from censored survival data
# AUC2 <- survivalROC(Stime=df$OS_time, status=df$OS_status,     
#                     marker = eta,     
#                     predict.time =  265, method="KM")
# AUC2$AUC
# # 0.6124465
# plot(AUC2$FP,
#      AUC2$TP,
#      type="l",
#      xlim=c(0,1), ylim=c(0,1),   
#      xlab=paste( "FP", "\n", "AUC = ",round(AUC2$AUC,3)),
#      ylab="TP",main="AUC2, Method = KM \n Year = 1")
# abline(0,1)






roc.fit <-timeROC(T = df$OS_time, # 结局时间
                  
                  delta = df$OS_status, # 生存结局
                  
                  marker = lpFit, # 预测变量
                  
                  cause = 1, # 阳性结局赋值，比如死亡，复发的赋值
                  
                  weighting = "marginal", ## 'marginal' = Kaplan-Meier estimator of the censoring distribution 
                  ## or 'cox' = censoring by the Cox model
                  
                  times = c(365*c(1,3,5)), ## 时间点，选取1年、3年和5年生存率
                  
                  ROC = T,
                  
                  iid = TRUE) 

library(tidyverse)

CI95 <- confint(roc.fit)[1] %>% unlist()

ROC_1 <- paste0(' : ',roc.fit$AUC[1] %>% as.numeric() %>% round(.,2),'(',CI95[1],'-',CI95[length(CI95)/2 + 1],')')
ROC_3 <- paste0(' : ',roc.fit$AUC[2] %>% as.numeric() %>% round(.,2),'(',CI95[2],'-',CI95[length(CI95)/2 + 2],')')
ROC_5 <- paste0(' : ',roc.fit$AUC[3] %>% as.numeric() %>% round(.,2),'(',CI95[3],'-',CI95[length(CI95)/2 + 3],')')

col_all <- rainbow(10)

filename <- paste0(paste0("./results/KM+timeROC/",project,"/",name[j],"_OS_AUC_t_group_Type.pdf"))


pdf(filename,onefile = F)

plot(roc.fit,time=365,col = col_all[1],add =FALSE,title = F)#time 是时间点，col是线条颜色，add指是否添加在上一张图中

plot(roc.fit,time=365*3,col = col_all[2],add = TRUE)

plot(roc.fit,time=365*5,col = col_all[3],add = TRUE)


title(main = 'Time-dependent ROC curve')

legend("bottomright",
       
       legend = c(paste0('AUC at 1 year',ROC_1), 
                  paste0('AUC at 3 years',ROC_3),
                  paste0('AUC at 5 years',ROC_5)),
       
       lty = c("solid","solid","solid","solid","solid"),
       col = col_all[1:5],
       bty = 'n')

dev.off()
}
}

load("D:/LUAD/LUAD_M6A/1116_LUAD_results_lasso_GSE72094/Hypoxia/Lasso/lasso_genes.RData")
load("D:/LUAD/LUAD_M6A/1116_LUAD_results_lasso_GSE72094/Hypoxia/Lasso/Final_genes.Rdata")
