rm(list = ls()) ## 清空环境变量
library("parallel")
library("randomForestSRC")
library("tidyverse")
library(tidyverse)
library(survivalsvm) ## 加载R包
getwd()
setwd("D:/LUAD/output")

load("D:/LUAD/data/TCGA_LUAD.Rdata")
#load("D:/LUAD/data/TCGA_LUAD_group.Rdata")
load("D:/LUAD/data/GSE30219.RData")
load("D:/LUAD/data/GSE31210.Rdata")
load("D:/LUAD/data/GSE72094.RData")

####0.单因素cox筛选 p <0.05
df <- TCGA_LUAD
BaSurv <- Surv(time = df$OS_time, ## 生存时间
               event = df$OS_status) ## 生存状态

UniCox <- function(x){ ## 构建一个R function 便于后期调用
  FML <- as.formula(paste0('BaSurv~',x)) ## 构建生存分析公式
  GCox <- coxph(FML, data = df) ## Cox分析
  GSum <- summary(GCox) ## 输出结果
  HR <- GSum$coefficients[,2] ## 输出HR值
  PValue <- GSum$coefficients[,5] ## 输出P值
  CI <- paste0(GSum$conf.int[,3:4],collapse = "-") ## 输出HR的执行区间
  coef <- GSum$coefficients[,1] %>% as.numeric()
  Unicox <- data.frame("characteristics" = x, ## 返回结果，并构建数据框
                       "coef" = coef,
                       "Hazard Ratio" = HR,
                       "CI95" = CI,
                       "P Value" = PValue)
  return(Unicox)
}

VarNames <- colnames(df)[-c(1,2)]## 输出需要分析的变量名字

UniVar <- lapply(VarNames,UniCox) ## 批量做Cox分析
UniVar <- ldply(UniVar,data.frame) ## 将结果整理为一个数据框 
save(UniVar,file = "D:/LUAD/output/单因素cox/UniVar_TCGA.Rdata")
#按照p=0.05筛选，可以自行调整筛选值
UniVar_0.05 <- UniVar$characteristics[which(UniVar$P.Value < 0.05)] %>% as.character()
UniVar_0.05<- read_csv(paste0("D:/LUAD/output/单因素cox/UniVar_0.05_TCGA.csv"))

#write.csv(UniVar_0.05,file = "D:/LUAD/output/单因素cox/UniVar_0.05_TCGA.csv",row.names=F)
save(UniVar_0.05,file = "D:/LUAD/output/单因素cox/UniVar_0.05.Rdata")

#5075 UniVar_0.05
#3738 UniVar_0.05

#处理缺氧基因集
genesetname<- "Hypoxia"
geneset<- read_csv(paste0("D:/LUAD/LUAD_M6A/特殊基因集/",genesetname,"/",genesetname,".csv")) ##去读特殊基因集合
geneset<-geneset$ENSEMBL
geneset_hypoxia_0.05 <- intersect(geneset,UniVar_0.05$x)
#write.csv(geneset_hypoxia_0.05,file = "D:/LUAD/output/单因素cox/UniVar_0.05_TCGA.csv",row.names=F)
geneset_hypoxia_0.05 <- read_csv("D:/LUAD/output/单因素cox/UniVar_0.05_TCGA.csv")
#save(geneset_hypoxia_0.05,file = paste0("D:/LUAD/output/geneset/hypoxia_0.05_TCGA.Rdata"))

#number = 62, hypoxia_UniVar_0.05


#一致性聚类 Consensus Clustering
#BiocManager::install("ConsensusClusterPlus")
library(ConsensusClusterPlus)
load(file = "D:/LUAD/output/geneset/hypoxia_0.05_TCGA.Rdata")
df_cliniclinfor <- GSE72094
#df_cliniclinfor <- TCGA_LUAD
df <- df_cliniclinfor[,-c(1,2)]
#1.0 准备数据  
## 行为特征，列为样本
df <- t(df)
mad(df[1, ])
mads=apply(df,1,mad)
#将数据先按索引排序-升序order(),然后在反过来,然后在按照反过来的索引顺序形成data
#df=df[rev(order(mads))[1:000],]
dim(df)
#归一化操作
df = sweep(df,1, apply(df,1,median,na.rm=T))
dim(df)
df[1:5,1:5]


#2.0 核心函数
#output_dir="D:/output"
setwd("D:/LUAD/output/Cluster")
maxK = 4
library(tidyverse)
df = as.data.frame(df)
df_cluster <- df %>% filter(rownames(df) %in% geneset_hypoxia_0.05$x )
df_cluster = t(t(df_cluster))
results = ConsensusClusterPlus(df_cluster,maxK=maxK,reps=50,pItem=0.8,pFeature=1, 
                               title="test_GSE72094",clusterAlg="pam",distance="minkowski",
                               innerLinkage = "complete",seed=102,plot="pdf", writeTable = TRUE)

#clusterAlg="hc",distance="canberra",'pearson': (1 - Pearson correlation), 
#'spearman' (1 - Spearman correlation), 'euclidean', 
#'binary', 'maximum', 'canberra', 'minkowski" or custom distance function.

#3.计算组间一致性和组类一致性
icl = calcICL(results,
             title="test_GSE72094",
             plot="pdf")

#4.PAC标准筛选K
Kvec = 2:maxK
x1 = 0.1; x2 = 0.9 # threshold defining the intermediate sub-interval
PAC = rep(NA,length(Kvec))
names(PAC) = paste("K=",Kvec,sep="") # from 2 to maxK

for(i in Kvec){
  M = results[[i]]$consensusMatrix
  Fn = ecdf(M[lower.tri(M)])
  PAC[i-1] = Fn(x2) - Fn(x1)
}#end for i

# The optimal K
optK = Kvec[which.min(PAC)]
optK
#最终选择 euclidean 50-0.364；同时KM曲线好

#K=2最好   hc: 70-0.777 60-0.75  50-0.741,piem=0.8  40-0752
#pam: pam 50\0.8 - 0.127  80-0.12   100-0.128  200-0.137
#minkowski"  50 - 0.364   euclidean 50-0.364


## output_dir 目录下生成不同K值下的聚类图和聚类评估图 
# 根据consensus CDF和Delta area图，选择最佳的k值：从K=2开始,计算K和K-1相比，
# CDF 曲线下面积的相对变化,选取增加不明显的点作为最佳的K值
# trackling plot:行为样本，列为每个K, 用热图展示样本在每个K下的cluster, 
# 用于定性评估不稳定的聚类和不稳定的样本


###!!!#
##5.亚型KM-plot
optK = 2
library(tidyverse)
table(results[[optK]]$consensusClass)
#1   2 
#189 314 
Cluster = results[[optK]]$consensusClass
identical(names(Cluster),rownames(df_cliniclinfor))
#添加亚型分组信息
## [1] TRUE
df_cliniclinfor$Cluster = Cluster
df_cliniclinfor_cluster = df_cliniclinfor
save(df_cliniclinfor_cluster,file =  "D:/LUAD/data/GSE31210_Cluster.Rdata")

df_new <- df_cliniclinfor_cluster[,c("OS_time","Cluster")]
write.csv(df_new,file = "D:/LUAD/data/GSE31210_Cluster.csv")
library(survival)
library(survminer)
sfit <- survfit(Surv(OS_time/365, OS_status)~Cluster,
                data = df_cliniclinfor)
#ggsurvplot(sfit,pval = T,palette = "jco")
p <- ggsurvplot(sfit, conf.int=F, pval=T, risk.table=T, palette = "jco",
                risk.table.height = 0.3)


pdf("./GSE31210_Hypoxia_group_KMplot.pdf",onefile = F,width = 8,height = 6)
print(p)
dev.off()


###### 6.PCA
#BiocManager::install("tinyarray")
#install.packages('factoextra')
#library(tinyarray)
exp <- as.data.frame(t(df_cliniclinfor[,-c(1,2)]))
exp <- exp[!duplicated(exp),]
mads <- apply(exp,1,mad)
#exp <- exp[rev(order(mads))[1:5000],]
exp <- t(df_cluster)
#par(mfrow = c(1,2))
#boxplot(df[,1:20],main = "before")
exp <-  sweep(exp,1, apply(exp,1,median,na.rm=T))
exp <- as.data.frame(exp)
#pca.plot = draw_pca(exp,Cluster,addEllipses = F)
save(pca.plot,file = "D:/LUAD/output/PCA/GSE31210_pcaplot.Rdata")

library(Rtsne)
tsne_out = Rtsne(exp,perplexity = 20,theta = 0,initial_dims = 50)
pdat = data.frame(tsne_out$Y,factor(Cluster))
colnames(pdat) = c("Cluster1","Cluster2","group")
head(pdat)
#Cluster1  Cluster2 group
#TCGA-05-4249-01A  10.431932  -2.335202     1
#TCGA-05-4250-01A  -3.505380   4.340155     2

library(ggplot2)
#BiocManager::install("paletteer")
library(paletteer)
pdf("D:/LUAD/output/PCA/GSE31210_PCA.pdf",onefile = F,width = 8,height = 6)
p <- ggplot(pdat,aes(Cluster1,Cluster2))+
  geom_point(aes(Cluster1,Cluster2,fill = group),shape = 21,color = "black")+
  stat_ellipse(aes(color = group,fill = group),
               geom = "polygon",
               alpha = 0.3,
               linetype = 2)+
  scale_color_paletteer_d("RColorBrewer::Set3")+
  scale_fill_paletteer_d("RColorBrewer::Set3")+
  theme_classic()+
  theme(legend.position = "top")
p

dev.off()


###7.临床桑基图
#install.packages("ggalluvial")
#载入包
library(ggplot2)
library(ggalluvial) 
library(xlsx)
data <- read.xlsx2('D:/LUAD/data/6-临床分析/TCGA_临床信息_cluster.xlsx', 1, header=TRUE,colClasses="character")
data <- data[,-c(1)]
df <- to_lodes_form(data[,1:ncol(data)],
                    axes = 1:ncol(data),
                    id = "value") 
#绘制桑基图（Sankey diagram
#col<- rep(c('#FCCEBE', '#99C3D9', '#89D7BC', '#FCF9F0', '#EDCDCE',
#            '#E2A5AD', '#9791A0', '#AFC7BF', '#9FBCC2', 'yellow'), 3)#自定义颜色

#新建一个PDF文件
pdf("D:/LUAD/output/桑基图/clinical_cluster.pdf",width = 8, height = 6)
#绘图
col <- rep(c("#A1DFAD","#FFB84F","#8BD7D9","#F09090","#D6C770","#81A2CD",
             "#25AAAC","#628BC1","#27A43F","#AA971E","#FFFFAD","#FDCDE6","#8A58BC","#7AD6CB","#BFB9DD","#A6E04C"),3)


#画图
#Sankey <- sankeyNetwork(Links = links, Nodes = nodes,
#                        Source = "IDsource", Target = "IDtarget", NodeGroup="group", LinkGroup="group",
#                        Value = "Value", NodeID = "name", colourScale = my_color, #nodePadding=10,
#                        sinksRight=FALSE, nodeWidth = 10,  fontSize=12,width=1000)

#Sankey


ggplot(df, aes(x = x, fill=stratum, label=stratum,
               stratum = stratum, alluvium  = value))+#数据
  geom_flow(width = 0.3,#连线宽度
            curve_type = "sine",#曲线形状，有linear、cubic、quintic、sine、arctangent、sigmoid几种类型可供调整
            alpha = 0.3,#透明度
            
            color = 'white',#间隔颜色
            size = 0.1)+#间隔宽度
  geom_stratum(width = 0.45)+#图中方块的宽度
  geom_text(stat = 'stratum', size = 4, color = 'black')+
  scale_fill_manual(values = col)+#自定义颜色
  theme_void()+#主题（无轴及网格线）
  theme(legend.position = 'none')#去除图例
dev.off() 

#突变 瀑布图
rm(list = ls())
#BiocManager::install("TCGAbiolinks",ask = F,update = F)
library(TCGAbiolinks)
library(maftools)

GDCquery_Maf(
  tumor = "LUAD",
  save.csv = TRUE,
  directory = "./download",
  pipelines = "mutect2"
)
getwd()
paad.clin <- GDCquery_clinic("TCGA-LUAD")
clin <- paad.clin %>% 
  dplyr::select(c(
    "submitter_id", "days_to_last_follow_up", "vital_status",
    "ajcc_pathologic_t", "ajcc_pathologic_n", "ajcc_pathologic_m",
    "ajcc_pathologic_stage", "gender"
  )) %>%
  `names<-`(c("Tumor_Sample_Barcode", "time", "status", "T", "N", "M", "stage", "gender"))


#mut <- GDCquery_Maf(tumor = "LUAD",pipelines = "mutect2",save.csv = TRUE,directory ="D:/LUAD/output/download") #下载突变数据
dim(mut)


###8. HRGs在两个亚型的表达水平


###9.GSVA
#BiocManager::install("GSVA")
library(tidyverse)
library(clusterProfiler)
library(msigdbr)  #install.packages("msigdbr")
library(GSVA) 
library(GSEABase)
library(pheatmap)
library(limma)
library(BiocParallel)
#setwd("")
## msigdbr包提取下载 一般尝试KEGG和GO做GSVA分析
##KEGG
KEGG_df_all <-  msigdbr(species = "Mus musculus", # Homo sapiens or Mus musculus
                        category = "C2",
                        subcategory = "CP:KEGG") 
KEGG_df <- dplyr::select(KEGG_df_all,gs_name,gs_exact_source,gene_symbol)
kegg_list <- split(KEGG_df$gene_symbol, KEGG_df$gs_name)
#
dat <- as.matrix(t(TCGA_LUAD[,-c(1,2)]))
dat_new <- as.data.frame(cbind(gene_id = rownames(dat),dat))
library(xlsx)
symbol <- read.xlsx2('D:/LUAD/data/gencode.gene.info.v22.xlsx', 1, header=TRUE,colClasses="character")
data_symbol     <- merge(dat_new,symbol,by = "gene_id",sort= F)
uid2  = data_symbol$gene_name[duplicated(data_symbol$gene_name)]

data_symbol <- data_symbol %>% filter(!gene_name %in% uid2)
rownames(data_symbol) <- data_symbol$gene_name
data_symbol <- data_symbol[,-c(1,505)]
data_symbol <- as.data.frame(data_symbol)
data_symbol_output <- dplyr::mutate_all(data_symbol,as.numeric)
data_symbol_output <- as.matrix(data_symbol_output)

#data_symbol <-as.matrix(data_symbol)
geneset <- kegg_list

gsva_mat <- gsva(expr=data_symbol_output, 
                 gset.idx.list=geneset, 
                 kcdf="Poisson" ,#"Gaussian" for logCPM,logRPKM,logTPM, "Poisson" for counts
                 verbose=T, 
                 parallel.sz = parallel::detectCores())#调用所有核

write.csv(gsva_mat,"D:/LUAD/output/GSVA/gsva_kegg_matrix.csv")
#通路的差异分析
cluster <- read.csv('D:/LUAD/data/TCGA_Cluster.csv', sep = ",")
colnames(cluster)[1] <- "Sample"
cluster <- cluster[,c(1,3)]
design <- model.matrix(~0+factor(cluster$Cluster))
#colnames(design) <- c("Cluster1","Cluster2")
colnames(design) <- levels(factor(cluster$Cluster))
rownames(design) <- colnames(gsva_mat)
comp <- 'Cluster1-Cluster2'
contrast.matrix <- makeContrasts(contrasts = c(comp), levels = design)

fit1 <- lmFit(gsva_mat,design)                 #拟合模型
fit2 <- contrasts.fit(fit1, contrast.matrix) #统计检验
efit <- eBayes(fit2)                         #修正

summary(decideTests(efit,lfc=1, p.value=0.1)) #统计查看差异结果
tempOutput <- topTable(efit, coef=c(comp), n=Inf)
degs <- na.omit(tempOutput) 
write.csv(degs,"D:/LUAD/output/GSVA/gsva_kegg_degs.results.csv")
#热图
padj_cutoff=0.05
log2FC_cutoff=log2(0.5)

keep <- rownames(degs[degs$adj.P.Val < padj_cutoff & abs(degs$logFC)>log2FC_cutoff, ])
length(keep)
dat <- gsva_mat[keep,] #选取前50进行展示

dat <- read.xlsx2('D:/LUAD/output/GSVA/gsva_dat.results.xlsx', 1, header=TRUE,colClasses="character")
rownames(dat) = dat[,1] 
dat <- dat[,-c(1)]
colnames(dat) <- gsub("\\.","-",colnames(dat))
dat <- dplyr::mutate_all(dat,as.numeric)
rownames(dat) = gsub("KEGG_","",rownames(dat))
dat = as.matrix(dat)

write.csv(dat,"D:/LUAD/output/GSVA/gsva_dat.results.csv")
setwd("D:/LUAD/output/")
cluster <- cluster[order(cluster[,2]),]
annotation_col = as.data.frame(cluster[,2])
rownames(annotation_col) = cluster[,1]
colnames(annotation_col) = "Cluster"


#利用colorRampPalette生成颜色函数b2p1，由绿色到红色的渐变
#b2p1 <- colorRampPalette(c("blue","white", "red"))
#anno_colors <- list(Group = c(Low = "#27549B", High = "#DF3B30"))
#生成25种颜色
#heatmap(data, cexCol = 1,scale="row",col = b2p1(25))
pheatmap::pheatmap(dat, 
                   scale = "row",
                   cluster_rows = T , cluster_cols = F ,
                   fontsize_row = 8,
                   color = colorRampPalette(c("#2756A6","White","#EC302B"))(256),
                   height = 3,
                   width=8,
                   annotation_col = annotation_col,
                   show_colnames = F,
                   show_rownames = T,
                   filename = paste0('D:/LUAD/output/GSVA/GSVA_go_heatmap.pdf'))

