
# Library
library(networkD3)
library(dplyr)

# Make a connection data frame
links <- data.frame(
  source=c("group_A","group_A", "group_B", "group_C", "group_C", "group_E"), 
  target=c("group_C","group_D", "group_E", "group_F", "group_G", "group_H"), 
  value=c(2,3, 2, 3, 1, 3)
)

# From these flows we need to create a node data frame: it lists every entities involved in the flow
nodes <- data.frame(
  name=c(as.character(links$source), as.character(links$target)) %>% 
    unique()
)

# With networkD3, connection must be provided using id, not using real name like in the links dataframe.. So we need to reformat it.
links$IDsource <- match(links$source, nodes$name)-1 
links$IDtarget <- match(links$target, nodes$name)-1

# prepare color scale: I give one specific color for each node.
my_color <- 'd3.scaleOrdinal() .domain(["group_A", "group_B","group_C", "group_D", "group_E", "group_F", "group_G", "group_H"]) .range(["blue", "blue" , "blue", "red", "red", "yellow", "purple", "purple"])'

# Make the Network. I call my colour scale with the colourScale argument
p <- sankeyNetwork(Links = links, Nodes = nodes, Source = "IDsource", Target = "IDtarget", 
                   Value = "value", NodeID = "name", colourScale=my_color)
p

# save the widget
# library(htmlwidgets)
# saveWidget(p, file=paste0( getwd(), "/HtmlWidget/sankeyColor1.html"))

# Add a 'group' column to the nodes data frame:
nodes$group <- as.factor(c("a","a","a","a","a","b","b","b"))

# Give a color for each group:
my_color <- 'd3.scaleOrdinal() .domain(["a", "b"]) .range(["#69b3a2", "steelblue"])'

# Make the Network
p <- sankeyNetwork(Links = links, Nodes = nodes, Source = "IDsource", Target = "IDtarget", 
                   Value = "value", NodeID = "name", 
                   colourScale=my_color, NodeGroup="group")
p

# save the widget
# library(htmlwidgets)
# saveWidget(p, file=paste0( getwd(), "/HtmlWidget/sankeyColor2.html"))



# Add a 'group' column to each connection:
links$group <- as.factor(c("type_a","type_a","type_a","type_b","type_b","type_b"))

# Add a 'group' column to each node. Here I decide to put all of them in the same group to make them grey
nodes$group <- as.factor(c("my_unique_group"))

# Give a color for each group:
my_color <- 'd3.scaleOrdinal() .domain(["type_a", "type_b", "my_unique_group"]) .range(["#69b3a2", "steelblue", "grey"])'

# Make the Network
p <- sankeyNetwork(Links = links, Nodes = nodes, Source = "IDsource", Target = "IDtarget", 
                   Value = "value", NodeID = "name", 
                   colourScale=my_color, LinkGroup="group", NodeGroup="group")

p

# save the widget
# library(htmlwidgets)
# saveWidget(p, file=paste0( getwd(), "/HtmlWidget/sankeyColor3.html"))






library(networkD3)
library(tidyverse)
library(RColorBrewer)

#读取刚才的数据框，给列命名
# df=read.table("test.txt",header = F,sep = "\t",stringsAsFactors = F)

links<- read.table("C:\\Users\\gaoka\\Desktop\\links.txt", 
              sep = "\t", header=T, check.names=FALSE)

# colnames(links)=c("Source","Target","Value")

#图形中的所有节点
nodes <- read.table("C:\\Users\\gaoka\\Desktop\\nodes.txt", 
                    sep = "\t", header=T, check.names=FALSE)

#将source和target重新编号
links$IDsource <- match(df$Source, nodes$name)-1
links$IDtarget <- match(df$Target, nodes$name)-1



# # Add a 'group' column to each connection:
# links$group <- as.factor(c("type_a","type_a","type_a","type_b","type_b","type_b"))
# 
# # Add a 'group' column to each node. Here I decide to put all of them in the same group to make them grey
# nodes$group <- as.factor(c("my_unique_group"))

# Give a color for each group:
my_color <- 'd3.scaleOrdinal() .domain(["I", "C", "H", "F", "M", "TypeA", "TypeB", "TypeC", "TypeD", "TypeE", "TypeF"]) .range(["#25AAAC","#628BC1","#27A43F","#AA971E","#8A58BC",
"#A1DFAD","#FFB84F","#8BD7D9","#F09090","#D6C770","#81A2CD"])'

#画图
Sankey <- sankeyNetwork(Links = links, Nodes = nodes,
                          Source = "IDsource", Target = "IDtarget", NodeGroup="group", LinkGroup="group",
                          Value = "Value", NodeID = "name", colourScale = my_color, #nodePadding=10,
                          sinksRight=FALSE, nodeWidth = 10,  fontSize=12,width=1000)

Sankey




#画出来的图形，还可以直接在R窗口中调整方块上下位置，不过还是建议保存为pdf后用AI调整
#nodeWidth每个节点方块的宽度
#nodePadding每一列节点方块之间在纵向上的间隔
#width是图形横向的宽度

#保存为pdf
library(htmlwidgets)
saveWidget(Sankey, file="C:\\Users\\gaoka\\Desktop\\NetworkMedicine_HERB_CMap\\Sankey.html")


library(webshot)
webshot("Sankey.html", "Sankey.pdf")







