# BiocManager::install("TCGAbiolinks")
library(TCGAbiolinks)
library(tidyverse)
library(maftools)
#计算每份样本的TMB
#TCGA_tmb <- read.csv('D:/LUAD/data/TMB/TCGA/TCGA.LUAD_all_mutect.somatic.maf.csv',sep = ",")
library(xlsx)
TCGA_tmb <- read.xlsx2('D:/LUAD/data/TMB/TCGA/TMB.xlsx', 1, header=TRUE,colClasses="character")
TCGA_tmb$TMB <- as.numeric(TCGA_tmb$TMB)
TCGA_tmb$Cluster <- as.factor(TCGA_tmb$Cluster)
data <- TCGA_tmb
# data$vriable=factor(data$group, levels=c("High","Low"))
#geom_bar表示画柱状图，position = "dodge"意思为将柱子并列放置，stat是指统计变换；stat='identity'是指不进行统计变换即业务量就是纵。
pdf('D:\\LUAD\\output\\LUAD_TMB/TMB.pdf', width = 3, height = 3.2)
p <- ggplot(data,aes(x = Cluster, y = TMB))+
  #stat_boxplot(geom="errorbar",width=0.3,size=0.5,position=position_dodge(0.6),color="black")+
  geom_boxplot( position=position_dodge(0.6),
                size=0.5,
                width=0.6,
                fill=c('#E21C21','#3A7CB5'),
                color="black",
                outlier.color = "black",
                outlier.fill = "black",
                outlier.shape = 19,
                outlier.size = 1.5,
                outlier.stroke = 0.5,
                outlier.alpha = 45,
                notch = F,
                notchwidth = 0.5)+
  geom_signif(comparisons=list(c("Cluster1", "Cluster2")),
              tip_length = 0.04, vjust=0.2) + 
  #geom_signif(comparisons=list(c("Cluster1", "Cluster2")),
  #            y_position = 3, tip_length = 0.04, vjust=0.2) +
  labs(title="", x="", y="TMB score")+
  theme(axis.title = element_text(size=18),
        axis.text = element_text(size=14))
p

dev.off()




#TCGA 的突变流程有 4 种，分别是：muse、varscan2、somaticsniper、mutect2
# 其中 muse 和 somaticsniper 只能计算点突变，无法识别 indel，所以不用考虑。
# 我们就以 PAAD 的 mutect 为例来进行说明吧
getwd()
setwd("D:/LUAD/data/TMB/")
GDCquery_Maf(
  tumor = "LUAD",
  save.csv = TRUE,
  directory = "./Downloads/TCGA",
  pipelines = "mutect2"
)



paad.clin <- GDCquery_clinic("TCGA-LUAD")
clin <- paad.clin %>% 
  dplyr::select(c(
    "submitter_id", "days_to_last_follow_up", "vital_status",
    "ajcc_pathologic_t", "ajcc_pathologic_n", "ajcc_pathologic_m",
    "ajcc_pathologic_stage", "gender"
  )) %>%
  `names<-`(c("Tumor_Sample_Barcode", "time", "status", "T", "N", "M", "stage", "gender"))

##根据cluster的high-low分组 修改table
load("D:/LUAD/data/TCGA_LUAD_group.Rdata")
# TCGA_LUAD$ID = rownames(TCGA_LUAD)
TCGA_ID <- rownames(TCGA_LUAD)

paad.maf <- read.csv("D:\\LUAD\\data\\TMB\\TCGA\\TCGA.LUAD_all_mutect.somatic.maf.csv")
paad.maf_new = transform(paad.maf, ID = substr(Tumor_Sample_Barcode, 1,16))

#intersect TCGA突变记录数 和TCGA_mRNA = 486个sample
paad.maf_new <- paad.maf_new[paad.maf_new$ID %in% TCGA_ID,]
paad.maf_new <- merge(paad.maf_new, TCGA_LUAD[,c("group","ID")], by="ID")

TCGA_LUAD_TMB_high <- paad.maf_new[paad.maf_new$group=="High",]
TCGA_LUAD_TMB_low <- paad.maf_new[paad.maf_new$group=="Low",]

TCGA_LUAD_TMB_high <- subset(TCGA_LUAD_TMB_high,select= -ID)
TCGA_LUAD_TMB_high <- subset(TCGA_LUAD_TMB_high,select= -group)


write.csv(TCGA_LUAD_TMB_high,file = "./TCGA_LUAD_TMB_high.csv")
write.csv(TCGA_LUAD_TMB_low,file = "./TCGA_LUAD_TMB_low.csv")
#需要使用7z压缩软件 存储为maf.gz格式


paad.maf <- read.maf(
  "D:/LUAD/LUAD_TMB/TCGA_LUAD_TMB_high.maf.gz",
  # clinicalData = clin,
  isTCGA = TRUE
)
# paad.maf_high = paad.maf
# data <- paad.maf@data
# paad.maf_high@datas = data
#读取数据框
# paad.maf_high <- data.table::as.data.table(
#   read.csv(file ="./TCGA_LUAD_TMB_high.rar",header = TRUE,sep = '\t',stringsAsFactors =FALSE, comment.char ="#"),
#   # clinicalData = clin,
#   isTCGA = TRUE
# )

#画图
vc_cols <- RColorBrewer::brewer.pal(n = 8, name = 'Paired')
names(vc_cols) <- c(
  'Frame_Shift_Del',
  'Missense_Mutation',
  'Nonsense_Mutation',
  'Multi_Hit',
  'Frame_Shift_Ins',
  'In_Frame_Ins',
  'Splice_Site',
  'In_Frame_Del'
)

#基因突变类型概况图
#生成png
#genesinfor
#png('./TCGA_TMB_low_genesinfor.png', height = 200, width = 350,units = "mm", res = 1000)
#p <- plotmafSummary(maf = paad.maf, rmOutlier = TRUE, addStat = 'median', dashboard = TRUE, titvRaw = FALSE, titleSize= c(1,1))
#dev.off()

#TMB
#生成pdf
pdf('./TCGA_TMB_high.pdf', height = 8, width = 12)
p <- oncoplot(maf = paad.maf, colors = vc_cols, top = 10,fontSize = 1,legendFontSize = 2,SampleNamefontSize = 1.5,titleFontSize=1.5 )
dev.off()
# load("D:\\LUAD\\LUAD_oearson\\2-Pearson Correlation\\6genes/6genes_cor_0.6.Rdata")




