# 调用R包
library(Rmisc) 
library(corrplot)
library(ggcorrplot)
library(RColorBrewer)
library(grDevices)
library(tidyverse)
#library(extrafont) # 导入外部字体
#library(extrafontdb) # 字体数据库
#library(Rttf2pt1)


rm(list=ls())
setwd("D:/LUAD/output/ssGSEA")
ssgsea.1 <- read.csv("D:/LUAD/data/ssGSEA_corrplot/ssgsea_corrplot.csv",sep = ",")
rownames(ssgsea.1) <- ssgsea.1[,1]
ssgsea.1 <- ssgsea.1[,-c(1)]
colnames(ssgsea.1) <- gsub("_",".",colnames(ssgsea.1))
ENV <- ssgsea.1
#3.1 计算相关性系数、置信区间和显著性检验
# 3.1.1 环境因子相关性分析，根据自己的数据选择相关性计算方法："pearson", "kendall", or "spearman"
#env.cor <- round(cor(ENV[4:14], method = "spearman"),3) # round(),对输出结果取小数点前三位,计算spearman相关性系数
#e?cor.mtestnv.cor <- round(cor(ENV[4:14], method = "kendall"),3) # round(),对输出结果取小数点前三位,计算kendall相关性系数
env.cor <- round(cor(ENV, method = "pearson"),3) # round(),对输出结果取小数点前三位,计算pearson相关性系数
env.cor
write.csv(env.cor,"./env.cor.csv")
# 3.1.2 进行显著性检验
#env.p <-round(cor_pmat(ENV[4:14],method = "spearman"),3) # spearman系数
#env.p <-round(cor_pmat(ENV[4:14],method = "kendall"),3) # kendall系数
env.p <-round(cor_pmat(ENV,method = "pearson"),3) # pearson系数
env.p 

# 3.1.3 使用corrplot的cor.mtest可以同时计算显著性检验和置信区间。返回是是一个列表
#cor.mtest(ENV[4:14], conf.level = 0.95)
env.uppCI = round(cor.mtest(ENV, conf.level = 0.95)$uppCI,3)
env.uppCI # 置信区间上界
env.lowCI = round(cor.mtest(ENV, conf.level = 0.95)$lowCI,3)
env.lowCI # 置信区间下界

# 3.2.3 三角形相关性热图
#?corrplot # 查看函数帮助手册，查看type参数可选值：full,upper或lower。
m=par(no.readonly = TRUE) # 保存默认图形设置参数
pdf("corrplot.pdf",width = 16,height = 12) # 保存图片到本地
par(mfrow=c(1,2)) # 形成1行、2列的图形矩阵
colors <- scales::alpha(colorRampPalette(c("white", "black"))(10),alpha = 0.1)
corrplot(corr =env.cor, p.mat = env.p,method = "circle",insig="label_sig",type = "upper",
         tl.cex=0.8, #字符大小
         pch.cex = 1.2,#用于设置显著性标签的字符大小。
         #col = colors,
         tl.col="black",tl.srt = 90,tl.offset=0.8)  #tloffset 距离,#insig="label_sig"  "*"显著性
#corrplot(corr =env.cor, p.mat = env.p,method = "circle",type = "lower")
dev.off()

###
#箱线图
####mytheme####
windowsFonts(myFont = windowsFont("Arial"))
###有x轴
mytheme <- theme_bw() + 
  theme(plot.title=element_text(family="myFont",size=rel(2.2),hjust=0.5),
        axis.title=element_text(family="myFont",size=rel(1.7)),
        axis.text=element_text(family="myFont",size=rel(1.5)),
        panel.grid.major=element_line(color="white"),
        panel.grid.minor=element_line(color="white"),
        panel.border=element_rect(color="black"),
        axis.line=element_line(color="gray",size=1),
        ###设置零轴
        # axis(side=2, pos=0, las=1, lty="dashed"),
        legend.title = element_blank()
  )


#分组信息的箱线图
ssgsea <- read.csv("D:/LUAD/data/ssGSEA_cluster/ssgsea.TCGA_output_箱线图.csv",sep = ",")
data <- ssgsea
rownames(data) <- data[,1]
data = data[,-c(1)]

data_all <- data %>% 
  pivot_longer(cols = !Cluster, 
               names_to = "celltype",
               values_to = "count")
data_all$celltype <- gsub("_",".",data_all$celltype)

#分组变量需要因子化
#data_all$Cluster <- factor(data_all$Cluster,levels = c("Cluster1","Cluster2"))
#data_all$celltype <-factor(data_all$celltype,levels = unique(data_all$celltype),ordered = F)

library(ggsignif)
library(ggpubr)
#geom_bar表示画柱状图，position = "dodge"意思为将柱子并列放置，stat是指统计变换；stat='identity'是指不进行统计变换即业务量就是纵。
pdf('D:\\LUAD\\output\\ssGSEA/TCGA_28immunecell_boxplot.pdf', width = 12, height = 6)
#my_compare <-list(c("Cluster1", "Cluster2"))
#data_all <- data.frame(Cluster = data_all$Cluster, celltype = data_all$celltype,
#                       count = as.numeric(data_all$count),stringsAsFactors = FALSE)
p <- ggplot(data_all,aes(x=celltype, y=count))+ 
    geom_boxplot(aes(fill= factor(Cluster)),width = 0.4,position = position_dodge(0.52),
                size=0.5,
                #width=0.6,
                #fill=c('#E21C21','#3A7CB5'),
                color="black",
                outlier.color = NA,
                outlier.fill = "black",
                outlier.shape = 19,
                outlier.size = 1.5,
                outlier.stroke = 0.5,
                outlier.alpha = 45,
                notch = F,
                notchwidth = 0.5,
                )+
  stat_compare_means(aes(group=Cluster),method = "t.test",label = "p.signif",hide.ns = F,cex=5, label.y = 1)+
  #wilcox
  #geom_signif(comparisons=my_compare,map_signif_level = TRUE, test = "t.test",
  #            tip_length = 0.1, vjust=0.2) + 
  labs(title="", x="", y="")+
  theme(
        axis.text.y = element_text(size=14),
        axis.text.x=element_text(size=10,angle=90,hjust=0.5,vjust=0.5),
        legend.key.size = unit(8, "pt"),legend.text=element_text(size=8),)
p
dev.off()
