library(ggplot2)
library(scales)
library(tidyverse)

# 下载28种免疫细胞的参考基因集 <http://cis.hku.hk/TISIDB/data/download/CellReports.txt>
rm(list=ls())
setwd("D:/LUAD/output/ssGSEA")
library(tidyverse)
options(stringsAsFactors = F)
geneSet <- read.csv("D:\\LUAD\\data\\ssGSEA_cluster\\CellMarker.txt",header = F,sep = "\t",) # 用EXCEL打开删除NA列
class(geneSet)

geneSet <- geneSet %>%
  column_to_rownames("V1")%>%t()
a <- geneSet
a <- a[1:nrow(a),]
set <- colnames(a)
l <- list()
#i <- "Activated CD8 T cell"
for (i in set) {
  x <-  as.character(a[,i])
  x <- x[nchar(x)!=0]
  x <-  as.character(x)
  l[[i]] <-x
}
save(l,file = "D:/LUAD/data/ssGSEA_cluster/gene_set.Rdata")

# 读取基因的表达矩阵
#rm(list=ls())
library(GSVA)
library(limma)
## 载入数据
library(xlsx)
data <- read.csv('D:/LUAD/data/TCGA_infor_Cluster.csv',  header=TRUE,sep = ",")
data <- data[!duplicated(data$锘縎ymbol),]
data <- data[-c(1),]
rownames(data) <- data[,1]
data <- data[,-c(1)]
data_output <- dplyr::mutate_all(data,as.numeric)

load(file = "D:/LUAD/data/ssGSEA_cluster/gene_set.Rdata")
dat <- as.matrix(data_output)#基因的表达量需要是矩阵，行为基因，列为样本
#开始进行ssGSEA
ssgsea<- gsva(dat, l,method='ssgsea',kcdf='Gaussian',abs.ranking=TRUE)

#基因集需要是list为对象。默认情况下，kcdf="Gaussian"，适用于输入表达式值连续的情况，如对数尺度的微阵列荧光单元、RNA-seq log-CPMs、log-RPKMs或log-TPMs。当输入表达式值是整数计数时，比如那些从RNA-seq实验中得到的值，那么这个参数应该设置为kcdf="Poisson"
# Min-Max标准化是指对原始数据进行线性变换，将值映射到[0，1]之间
# 这里是将每个样本中不同的免疫细胞比例标准化到0-1之间
ssgsea.1 <- ssgsea
for (i in colnames(ssgsea)) {
  #i <- colnames(ssgsea)[1]
  ssgsea.1[,i] <- (ssgsea[,i] -min(ssgsea[,i]))/(max(ssgsea[,i] )-min(ssgsea[,i] ))
  
}
apply(ssgsea.1[,1:6], 2, range)
colnames(ssgsea.1) <- gsub("\\.","-",colnames(ssgsea.1))



ssgsea.1 <- read.csv("D:/LUAD/data/ssGSEA_cluster/ssgsea.TCGA_临床信息_箱线图_results.csv",sep = ",")
rownames(ssgsea.1) <- ssgsea.1[,1]
ssgsea.1 <- ssgsea.1[,-c(1)]

ssgsea_head = as.data.frame(t(ssgsea.1))
ssgsea_head = ssgsea_head[,c(1:7)]
ssgsea_head$Age = as.numeric(ssgsea_head$Age)

ssgsea.1_heat <- ssgsea.1[-c(1:7),]
ssgsea.1_heat <- dplyr::mutate_all(ssgsea.1_heat,as.numeric)
ssgsea.1_heat = as.matrix(ssgsea.1_heat)
#热图
library(pheatmap)
#BiocManager::install("ComplexHeatmap")
library(ComplexHeatmap)
library(circlize)
col_fun2 <- colorRamp2(
  c(20, 60, 100),  #根据值的范围设置
  c("#ff7f00", "white", "#1f78b4")
)

ha <- HeatmapAnnotation(
  Age = ssgsea_head$Age,
  Gender = ssgsea_head$Gender,
  Clinical_M = ssgsea_head$Clinical_M,
  Clinical_N = ssgsea_head$Clinical_N,
  Clinical_T = ssgsea_head$Clinical_T,
  Status = ssgsea_head$Status,
  Cluster = ssgsea_head$Cluster,
  col = list( 
    Age = col_fun2 , #连续
    Gender =  c("Male" = "#7AD889","Female" = "#D2B31D"),
    Clinical_M = c( "M0" = "#BD79F9", "M1" = "#9043E7","MX"= "#621CA4"),
    Clinical_N = c( "N0" = "#F7E4D0", "N1" = "#E1AF86", "N2" = "#D27F3F", "N3" = "#AA5F25", "NX" = "#664125"),
    Clinical_T = c( "T1" = "#F7E2A4", "T2" = "#F4DC64","T3"= "#EDC42C","T4"= "#B19519","TX"= "#735F13"),
    Status = c("Alive" = "#D5F2E9", "Dead" = "#6893E9"),
    Cluster = c("Cluster1" = "#DE3759","Cluster2" = "#5175E0") #分类
  )
)

#p <- pheatmap::pheatmap(
#  ssgsea.1_heat, 
#  cluster_cols = F,
#  top_annotation = ha,
#  show_colnames = F,
#  fontsize=5
#)

pdf("D:/LUAD/output/ssGSEA/ssGSEA.pdf",onefile = F,width = 8,height = 6)
p <- pheatmap(ssgsea.1_heat,
              show_colnames = F,
              cluster_rows = T,
              cluster_cols = F,
              top_annotation = ha,
              fontsize=5)#画热图


print(p)
dev.off()
